@@ -1,16 +1,22 @@
 use anyhow::{Context, Error, Result};
 use bstr::ByteSlice;
 use env_logger::Env;
-use grep_cli::{stdout, DecompressionReaderBuilder};
+use grep_cli::stdout;
 use hcklib::{
-    core::Core,
+    core::{Core, CoreConfig, CoreConfigBuilder, HckInput},
     field_range::{FieldRange, RegexOrStr},
+    line_parser::{RegexLineParser, SubStrLineParser},
+    mmap::MmapChoice,
 };
 use log::error;
 use regex::bytes::Regex;
+use ripline::{
+    line_buffer::{LineBuffer, LineBufferBuilder},
+    LineTerminator,
+};
 use std::{
     fs::File,
-    io::{self, BufRead, BufReader, BufWriter, Read, Write},
+    io::{self, BufWriter, Write},
     path::{Path, PathBuf},
     process::exit,
 };
@@ -45,31 +51,6 @@ pub mod built_info {
         pub static ref VERSION: String = get_software_version();
     }
 }
-/// Determine whether we should read from a file or stdin.
-fn select_input<P: AsRef<Path>>(path: P, try_decompress: bool) -> Result<Box<dyn Read>> {
-    let reader: Box<dyn Read> =
-        if path.as_ref().as_os_str() == "-" {
-            get_stdin()
-        } else if try_decompress {
-            Box::new(
-                DecompressionReaderBuilder::new()
-                    .build(&path)
-                    .with_context(|| {
-                        format!("Failed to open {} for reading", path.as_ref().display())
-                    })?,
-            )
-        } else {
-            Box::new(File::open(&path).with_context(|| {
-                format!("Failed to open {} for reading", path.as_ref().display())
-            })?)
-        };
-    Ok(reader)
-}
-
-#[inline]
-fn get_stdin() -> Box<dyn Read> {
-    Box::new(io::stdin())
-}
 
 /// Determine if we should write to a file or stdout.
 fn select_output<P: AsRef<Path>>(output: Option<P>) -> Result<Box<dyn Write>> {
@@ -171,27 +152,63 @@ struct Opts {
     /// Try to find the correct decompression method based on the file extensions
     #[structopt(short = "z", long)]
     try_decompress: bool,
+
+    /// Disallow the posibility of using mmap
+    #[structopt(long)]
+    no_mmap: bool,
+
+    /// Support CRLF newlines
+    #[structopt(long)]
+    crlf: bool,
 }
 
 fn main() -> Result<()> {
-    // TODO: add the complement argument to flip the FieldRange / excludes
-    // TODO: parameterize newline
+    // TODO: move tests / add more tests
     env_logger::Builder::from_env(Env::default().default_filter_or("info")).init();
     let opts = Opts::from_args();
+
     let mut writer = select_output(opts.output.as_ref())?;
 
-    let readers: Vec<Result<Box<dyn Read>>> = if opts.input.is_empty() {
-        vec![Ok(get_stdin())]
+    let inputs: Vec<HckInput<PathBuf>> = if opts.input.is_empty() {
+        vec![HckInput::Stdin]
     } else {
         opts.input
             .iter()
-            .map(|p| select_input(p, opts.try_decompress))
+            .map(|p| {
+                if p.as_os_str() == "-" {
+                    HckInput::Stdin
+                } else {
+                    HckInput::Path(p.clone())
+                }
+            })
             .collect()
     };
 
-    for r in readers {
-        let mut r = r?;
-        if let Err(err) = run(&mut r, &mut writer, &opts) {
+    let mut conf_builder = CoreConfigBuilder::new();
+
+    let line_term = if opts.crlf {
+        LineTerminator::crlf()
+    } else {
+        LineTerminator::default()
+    };
+    conf_builder.line_terminator(line_term);
+
+    let mmap = if opts.no_mmap {
+        MmapChoice::never()
+    } else {
+        unsafe { MmapChoice::auto() }
+    };
+    conf_builder.mmap(mmap);
+    conf_builder.delimiter(&opts.delimiter.as_bytes());
+    conf_builder.output_delimiter(&opts.output_delimiter.as_bytes());
+    conf_builder.is_regex_parser(opts.delim_is_regex);
+    conf_builder.try_decompress(opts.try_decompress);
+    let conf = conf_builder.build();
+
+    let mut line_buffer = LineBufferBuilder::new().build();
+
+    for input in inputs.into_iter() {
+        if let Err(err) = run(input, &mut writer, &opts, conf, &mut line_buffer) {
             if is_broken_pipe(&err) {
                 exit(0)
             }
@@ -203,60 +220,84 @@ fn main() -> Result<()> {
 }
 
 /// Run the actual parsing and writing
-fn run<R: Read, W: Write>(reader: &mut R, writer: &mut W, opts: &Opts) -> Result<()> {
-    let mut writer = BufWriter::new(writer);
-    let mut reader = BufReader::new(reader);
-    reader.fill_buf()?;
-    let first_line = reader
-        .buffer()
-        .find_byte(b'\n')
-        .expect("no first line found");
+fn run<W: Write>(
+    input: HckInput<PathBuf>,
+    writer: &mut W,
+    opts: &Opts,
+    conf: CoreConfig,
+    line_buffer: &mut LineBuffer,
+) -> Result<()> {
+    let writer = BufWriter::new(writer);
 
-    let delim = if opts.delim_is_regex {
-        RegexOrStr::Regex(Regex::new(&opts.delimiter)?)
+    let delim = if conf.is_parser_regex() {
+        RegexOrStr::Regex(Regex::new(conf.delimiter().to_str()?)?)
     } else {
-        RegexOrStr::Str(&opts.delimiter)
+        RegexOrStr::Str(conf.delimiter().to_str()?)
     };
 
-    let fields = match (&opts.fields, &opts.header_fields) {
+    // Parser the fields in the context of the files being looked at
+    let (extra, fields) = match (&opts.fields, &opts.header_fields) {
         (Some(field_list), Some(header_fields)) => {
+            let first_line = input.peek_first_line()?;
             let mut fields = FieldRange::from_list(field_list)?;
             let header_fields = FieldRange::from_header_list(
                 header_fields,
-                &reader.buffer()[..first_line - 1],
+                first_line.as_bytes(),
                 &delim,
                 opts.header_is_regex,
             )?;
             fields.extend(header_fields.into_iter());
             FieldRange::post_process_ranges(&mut fields);
-            fields
+            (Some(first_line), fields)
+        }
+        (Some(field_list), None) => (None, FieldRange::from_list(field_list)?),
+        (None, Some(header_fields)) => {
+            let first_line = input.peek_first_line()?;
+            let fields = FieldRange::from_header_list(
+                header_fields,
+                first_line.as_bytes(),
+                &delim,
+                opts.header_is_regex,
+            )?;
+            (Some(first_line), fields)
         }
-        (Some(field_list), None) => FieldRange::from_list(field_list)?,
-        (None, Some(header_fields)) => FieldRange::from_header_list(
-            header_fields,
-            &reader.buffer()[..first_line - 1],
-            &delim,
-            opts.header_is_regex,
-        )?,
         (None, None) => {
             eprintln!("Must select one or both `fields` and 'header-fields`.");
             exit(1);
         }
     };
 
-    let mut core = Core::new(&mut writer, &opts.output_delimiter.as_bytes(), &fields);
-
     match &delim {
-        RegexOrStr::Regex(regex) => core.process_reader_regex(&mut reader, &regex)?,
-        RegexOrStr::Str(s) => core.process_reader_substr(&mut reader, s.as_bytes())?,
-    }
+        RegexOrStr::Regex(regex) => {
+            let mut core = Core::new(
+                &conf,
+                &fields,
+                RegexLineParser::new(&fields, &regex),
+                line_buffer,
+            );
+            core.hck_input(input, writer, extra)?;
+        }
+        RegexOrStr::Str(s) => {
+            let mut core = Core::new(
+                &conf,
+                &fields,
+                SubStrLineParser::new(&fields, s.as_bytes()),
+                line_buffer,
+            );
+            core.hck_input(input, writer, extra)?;
+        }
+    };
     Ok(())
 }
 
 #[cfg(test)]
 mod test {
+
+    use std::io::BufReader;
+
     use super::*;
     use bstr::io::BufReadExt;
+    use rstest::rstest;
     use tempfile::TempDir;
 
     /// Build a set of opts for testing
@@ -264,6 +305,7 @@ mod test {
         input_file: impl AsRef<Path>,
         output_file: impl AsRef<Path>,
         fields: &str,
+        no_mmap: bool,
     ) -> Opts {
         Opts {
             input: vec![input_file.as_ref().to_path_buf()],
@@ -275,6 +317,8 @@ mod test {
             header_fields: None,
             header_is_regex: true,
             try_decompress: false,
+            no_mmap,
+            crlf: false,
         }
     }
 
@@ -283,6 +327,7 @@ mod test {
         input_file: impl AsRef<Path>,
         output_file: impl AsRef<Path>,
         fields: &str,
+        no_mmap: bool,
     ) -> Opts {
         Opts {
             input: vec![input_file.as_ref().to_path_buf()],
@@ -294,6 +339,8 @@ mod test {
             header_fields: None,
             header_is_regex: true,
             try_decompress: false,
+            no_mmap,
+            crlf: false,
         }
     }
 
@@ -325,21 +372,37 @@ mod test {
 
     // Wrap the run function to create the readers and writers.
     fn run_wrapper<P: AsRef<Path>>(input: P, output: P, opts: &Opts) {
-        let mut reader = BufReader::new(File::open(input).unwrap());
+        let conf = CoreConfigBuilder::new()
+            .delimiter(opts.delimiter.as_bytes())
+            .is_regex_parser(opts.delim_is_regex)
+            .mmap(if opts.no_mmap {
+                MmapChoice::never()
+            } else {
+                unsafe { MmapChoice::auto() }
+            })
+            .output_delimiter(opts.output_delimiter.as_bytes())
+            .build();
+        let mut line_buffer = LineBufferBuilder::new().build();
         let mut writer = BufWriter::new(File::create(output).unwrap());
-        // let mut writer = BufferedOutput::new(writer, FLUSH_SIZE, RESERVE_SIZE, MAX_SIZE);
-        run(&mut reader, &mut writer, opts).unwrap();
+        run(
+            HckInput::Path(input.as_ref().to_owned()),
+            &mut writer,
+            opts,
+            conf,
+            &mut line_buffer,
+        )
+        .unwrap();
     }
 
     const FOURSPACE: &str = "    ";
 
-    #[test]
+    #[rstest]
     #[rustfmt::skip::macros(vec)]
-    fn test_read_single_values() {
+    fn test_read_single_values(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "1");
+        let opts = build_opts(&input_file, &output_file, "1", no_mmap);
         let data = vec![
             vec!["a", "b", "c"],
             vec!["1", "2", "3"],
@@ -351,12 +414,12 @@ mod test {
         assert_eq!(filtered, vec![vec!["a"], vec!["1"]]);
     }
 
-    #[test]
-    fn test_read_several_single_values() {
+    #[rstest]
+    fn test_read_several_single_values(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "1,3");
+        let opts = build_opts(&input_file, &output_file, "1,3", no_mmap);
         let data = vec![vec!["a", "b", "c"], vec!["1", "2", "3"]];
         write_file(&input_file, data, FOURSPACE);
         run_wrapper(&input_file, &output_file, &opts);
@@ -365,12 +428,12 @@ mod test {
         assert_eq!(filtered, vec![vec!["a", "c"], vec!["1", "3"]]);
     }
 
-    #[test]
-    fn test_read_several_single_values_with_invalid_utf8() {
+    #[rstest]
+    fn test_read_several_single_values_with_invalid_utf8(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "1,3");
+        let opts = build_opts(&input_file, &output_file, "1,3", no_mmap);
         let bad_str = unsafe { String::from_utf8_unchecked(b"a\xED\xA0\x80z".to_vec()) };
         let data = vec![vec![bad_str.as_str(), "b", "c"], vec!["1", "2", "3"]];
         write_file(&input_file, data, FOURSPACE);
@@ -380,12 +443,12 @@ mod test {
         assert_eq!(filtered, vec![vec![bad_str.as_str(), "c"], vec!["1", "3"]]);
     }
 
-    #[test]
-    fn test_read_single_range() {
+    #[rstest]
+    fn test_read_single_range(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "2-");
+        let opts = build_opts(&input_file, &output_file, "2-", no_mmap);
         let data = vec![vec!["a", "b", "c", "d"], vec!["1", "2", "3", "4"]];
         write_file(&input_file, data, FOURSPACE);
         run_wrapper(&input_file, &output_file, &opts);
@@ -394,12 +457,12 @@ mod test {
         assert_eq!(filtered, vec![vec!["b", "c", "d"], vec!["2", "3", "4"]]);
     }
 
-    #[test]
-    fn test_read_serveral_range() {
+    #[rstest]
+    fn test_read_serveral_range(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "2-4,6-");
+        let opts = build_opts(&input_file, &output_file, "2-4,6-", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -414,12 +477,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_read_mixed_fields1() {
+    #[rstest]
+    fn test_read_mixed_fields1(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "2,4-");
+        let opts = build_opts(&input_file, &output_file, "2,4-", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -434,12 +497,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_read_mixed_fields2() {
+    #[rstest]
+    fn test_read_mixed_fields2(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "-4,7");
+        let opts = build_opts(&input_file, &output_file, "-4,7", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -454,12 +517,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_read_no_delimis_found() {
+    #[rstest]
+    fn test_read_no_delimis_found(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "-4,7");
+        let opts = build_opts(&input_file, &output_file, "-4,7", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -473,12 +536,12 @@ mod test {
         assert_eq!(filtered, vec![vec!["a-b-c-d-e-f-g"], vec!["1-2-3-4-5-6-7"]]);
     }
 
-    #[test]
-    fn test_read_over_end() {
+    #[rstest]
+    fn test_read_over_end(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "-4,8,11-");
+        let opts = build_opts(&input_file, &output_file, "-4,8,11-", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -494,12 +557,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_reorder1() {
+    #[rstest]
+    fn test_reorder1(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts(&input_file, &output_file, "6,-4");
+        let opts = build_opts(&input_file, &output_file, "6,-4", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -515,13 +578,13 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_reorder2() {
+    #[rstest]
+    fn test_reorder2(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
         // 4-5 should not be repeated at the end and only written once.
-        let opts = build_opts(&input_file, &output_file, "3-,1,4-5");
+        let opts = build_opts(&input_file, &output_file, "3-,1,4-5", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -540,13 +603,13 @@ mod test {
         );
     }
 
-    #[test]
+    #[rstest]
     #[rustfmt::skip::macros(vec)]
-    fn test_read_single_values_not_regex() {
+    fn test_read_single_values_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "1");
+        let opts = build_opts_not_regex(&input_file, &output_file, "1", no_mmap);
         let data = vec![
             vec!["a", "b", "c"],
             vec!["1", "2", "3"],
@@ -558,12 +621,12 @@ mod test {
         assert_eq!(filtered, vec![vec!["a"], vec!["1"]]);
     }
 
-    #[test]
-    fn test_read_several_single_values_not_regex() {
+    #[rstest]
+    fn test_read_several_single_values_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "1,3");
+        let opts = build_opts_not_regex(&input_file, &output_file, "1,3", no_mmap);
         let data = vec![vec!["a", "b", "c"], vec!["1", "2", "3"]];
         write_file(&input_file, data, FOURSPACE);
         run_wrapper(&input_file, &output_file, &opts);
@@ -572,12 +635,14 @@ mod test {
         assert_eq!(filtered, vec![vec!["a", "c"], vec!["1", "3"]]);
     }
 
-    #[test]
-    fn test_read_several_single_values_with_invalid_utf8_not_regex() {
+    #[rstest]
+    fn test_read_several_single_values_with_invalid_utf8_not_regex(
+        #[values(true, false)] no_mmap: bool,
+    ) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "1,3");
+        let opts = build_opts_not_regex(&input_file, &output_file, "1,3", no_mmap);
         let bad_str = unsafe { String::from_utf8_unchecked(b"a\xED\xA0\x80z".to_vec()) };
         let data = vec![vec![bad_str.as_str(), "b", "c"], vec!["1", "2", "3"]];
         write_file(&input_file, data, FOURSPACE);
@@ -587,12 +652,12 @@ mod test {
         assert_eq!(filtered, vec![vec![bad_str.as_str(), "c"], vec!["1", "3"]]);
     }
 
-    #[test]
-    fn test_read_single_range_not_regex() {
+    #[rstest]
+    fn test_read_single_range_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "2-");
+        let opts = build_opts_not_regex(&input_file, &output_file, "2-", no_mmap);
         let data = vec![vec!["a", "b", "c", "d"], vec!["1", "2", "3", "4"]];
         write_file(&input_file, data, FOURSPACE);
         run_wrapper(&input_file, &output_file, &opts);
@@ -601,12 +666,12 @@ mod test {
         assert_eq!(filtered, vec![vec!["b", "c", "d"], vec!["2", "3", "4"]]);
     }
 
-    #[test]
-    fn test_read_serveral_range_not_regex() {
+    #[rstest]
+    fn test_read_serveral_range_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "2-4,6-");
+        let opts = build_opts_not_regex(&input_file, &output_file, "2-4,6-", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -621,12 +686,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_read_mixed_fields1_not_regex() {
+    #[rstest]
+    fn test_read_mixed_fields1_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "2,4-");
+        let opts = build_opts_not_regex(&input_file, &output_file, "2,4-", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -641,12 +706,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_read_mixed_fields2_not_regex() {
+    #[rstest]
+    fn test_read_mixed_fields2_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7");
+        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -661,12 +726,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_read_no_delimis_found_not_regex() {
+    #[rstest]
+    fn test_read_no_delimis_found_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7");
+        let opts = build_opts_not_regex(&input_file, &output_file, "-4,7", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -680,12 +745,12 @@ mod test {
         assert_eq!(filtered, vec![vec!["a-b-c-d-e-f-g"], vec!["1-2-3-4-5-6-7"]]);
     }
 
-    #[test]
-    fn test_read_over_end_not_regex() {
+    #[rstest]
+    fn test_read_over_end_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "-4,8,11-");
+        let opts = build_opts_not_regex(&input_file, &output_file, "-4,8,11-", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -701,12 +766,12 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_reorder1_not_regex() {
+    #[rstest]
+    fn test_reorder1_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
-        let opts = build_opts_not_regex(&input_file, &output_file, "6,-4");
+        let opts = build_opts_not_regex(&input_file, &output_file, "6,-4", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
@@ -722,13 +787,13 @@ mod test {
         );
     }
 
-    #[test]
-    fn test_reorder2_not_regex() {
+    #[rstest]
+    fn test_reorder2_not_regex(#[values(true, false)] no_mmap: bool) {
         let tmp = TempDir::new().unwrap();
         let input_file = tmp.path().join("input.txt");
         let output_file = tmp.path().join("output.txt");
         // 4-5 should not be repeated at the end and only written once.
-        let opts = build_opts_not_regex(&input_file, &output_file, "3-,1,4-5");
+        let opts = build_opts_not_regex(&input_file, &output_file, "3-,1,4-5", no_mmap);
         let data = vec![
             vec!["a", "b", "c", "d", "e", "f", "g"],
             vec!["1", "2", "3", "4", "5", "6", "7"],
